"use strict";

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

/* eslint-disable object-curly-newline, no-underscore-dangle, camelcase, no-unneeded-ternary */
var qs = require('qs');

var EventEmitter = require('events');

var helpers = require('../utils/helpers');

var RequestQueue = require('../utils/request-queue');

function applyParams(parameters, options) {
  var apiKey = options.apiKey,
      version = options.version,
      sessionId = options.sessionId,
      clientId = options.clientId,
      userId = options.userId,
      segments = options.segments,
      testCells = options.testCells,
      requestMethod = options.requestMethod,
      beaconMode = options.beaconMode;

  var _helpers$getWindowLoc = helpers.getWindowLocation(),
      host = _helpers$getWindowLoc.host,
      pathname = _helpers$getWindowLoc.pathname;

  var sendReferrerWithTrackingEvents = options.sendReferrerWithTrackingEvents === false ? false : true; // Defaults to 'true'

  var aggregateParams = Object.assign(parameters);

  if (version) {
    aggregateParams.c = version;
  }

  if (clientId) {
    aggregateParams.i = clientId;
  }

  if (sessionId) {
    aggregateParams.s = sessionId;
  }

  if (userId) {
    aggregateParams.ui = userId;
  }

  if (segments && segments.length) {
    aggregateParams.us = segments;
  }

  if (apiKey) {
    aggregateParams.key = apiKey;
  }

  if (testCells) {
    Object.keys(testCells).forEach(function (testCellKey) {
      aggregateParams["ef-".concat(testCellKey)] = testCells[testCellKey];
    });
  }

  if (beaconMode && requestMethod && requestMethod.match(/POST/i)) {
    aggregateParams.beacon = true;
  }

  if (sendReferrerWithTrackingEvents && host) {
    aggregateParams.origin_referrer = host;

    if (pathname) {
      aggregateParams.origin_referrer += pathname;
    }
  }

  aggregateParams._dt = Date.now();
  aggregateParams = helpers.cleanParams(aggregateParams);
  return aggregateParams;
} // Append common parameters to supplied parameters object and return as string


function applyParamsAsString(parameters, options) {
  return qs.stringify(applyParams(parameters, options), {
    indices: false
  });
}
/**
 * Interface to tracking related API calls
 *
 * @module tracker
 * @inner
 * @returns {object}
 */


var Tracker = /*#__PURE__*/function () {
  function Tracker(options) {
    _classCallCheck(this, Tracker);

    this.options = options || {};
    this.eventemitter = new EventEmitter();
    this.requests = new RequestQueue(options, this.eventemitter);
  }
  /**
   * Send session start event to API
   *
   * @function trackSessionStart
   * @param {object} [networkParameters] - Parameters relevant to the network request
   * @param {number} [networkParameters.timeout] - Request timeout (in milliseconds)
   * @returns {(true|Error)}
   * @example
   * constructorio.tracker.trackSessionStart();
   */


  _createClass(Tracker, [{
    key: "trackSessionStart",
    value: function trackSessionStart() {
      var networkParameters = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var url = "".concat(this.options.serviceUrl, "/behavior?");
      var queryParams = {
        action: 'session_start'
      };
      this.requests.queue("".concat(url).concat(applyParamsAsString(queryParams, this.options)), undefined, undefined, networkParameters);
      this.requests.send();
      return true;
    }
    /**
     * Send input focus event to API
     *
     * @function trackInputFocus
     * @param {object} [networkParameters] - Parameters relevant to the network request
     * @param {number} [networkParameters.timeout] - Request timeout (in milliseconds)
     * @returns {(true|Error)}
     * @description User focused on search input element
     * @example
     * constructorio.tracker.trackInputFocus();
     */

  }, {
    key: "trackInputFocus",
    value: function trackInputFocus() {
      var networkParameters = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var url = "".concat(this.options.serviceUrl, "/behavior?");
      var queryParams = {
        action: 'focus'
      };
      this.requests.queue("".concat(url).concat(applyParamsAsString(queryParams, this.options)), undefined, undefined, networkParameters);
      this.requests.send();
      return true;
    }
    /**
     * Send autocomplete select event to API
     *
     * @function trackAutocompleteSelect
     * @param {string} term - Term of selected autocomplete item
     * @param {object} parameters - Additional parameters to be sent with request
     * @param {string} parameters.original_query - The current autocomplete search query
     * @param {string} parameters.section - Section the selected item resides within
     * @param {string} [parameters.tr] - Trigger used to select the item (click, etc.)
     * @param {string} [parameters.group_id] - Group identifier of selected item
     * @param {string} [parameters.display_name] - Display name of group of selected item
     * @param {object} [networkParameters] - Parameters relevant to the network request
     * @param {number} [networkParameters.timeout] - Request timeout (in milliseconds)
     * @returns {(true|Error)}
     * @description User selected (clicked, or navigated to via keyboard) a result that appeared within autocomplete
     * @example
     * constructorio.tracker.trackAutocompleteSelect('T-Shirt', {
     *   original_query: 'Shirt',
     *   section: 'Products',
     *   tr: 'click',
     *   group_id: '88JU230',
     *   display_name: 'apparel',
     * });
     */

  }, {
    key: "trackAutocompleteSelect",
    value: function trackAutocompleteSelect(term, parameters) {
      var networkParameters = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

      // Ensure term is provided (required)
      if (term && typeof term === 'string') {
        // Ensure parameters are provided (required)
        if (parameters && _typeof(parameters) === 'object' && !Array.isArray(parameters)) {
          var url = "".concat(this.options.serviceUrl, "/autocomplete/").concat(helpers.ourEncodeURIComponent(term), "/select?");
          var queryParams = {};
          var original_query = parameters.original_query,
              section = parameters.section,
              original_section = parameters.original_section,
              tr = parameters.tr,
              group_id = parameters.group_id,
              display_name = parameters.display_name;

          if (original_query) {
            queryParams.original_query = original_query;
          }

          if (tr) {
            queryParams.tr = tr;
          }

          if (original_section || section) {
            queryParams.section = original_section || section;
          }

          if (group_id) {
            queryParams.group = {
              group_id: group_id,
              display_name: display_name
            };
          }

          this.requests.queue("".concat(url).concat(applyParamsAsString(queryParams, this.options)), undefined, undefined, networkParameters);
          this.requests.send();
          return true;
        }

        this.requests.send();
        return new Error('parameters are required of type object');
      }

      this.requests.send();
      return new Error('term is a required parameter of type string');
    }
    /**
     * Send autocomplete search event to API
     *
     * @function trackSearchSubmit
     * @param {string} term - Term of submitted autocomplete event
     * @param {object} parameters - Additional parameters to be sent with request
     * @param {string} parameters.original_query - The current autocomplete search query
     * @param {string} [parameters.group_id] - Group identifier of selected item
     * @param {string} [parameters.display_name] - Display name of group of selected item
     * @param {object} [networkParameters] - Parameters relevant to the network request
     * @param {number} [networkParameters.timeout] - Request timeout (in milliseconds)
     * @returns {(true|Error)}
     * @description User submitted a search (pressing enter within input element, or clicking submit element)
     * @example
     * constructorio.tracker.trackSearchSubmit('T-Shirt', {
     *   original_query: 'Shirt',
     *   group_id: '88JU230',
     *   display_name: 'apparel',
     * });
     */

  }, {
    key: "trackSearchSubmit",
    value: function trackSearchSubmit(term, parameters) {
      var networkParameters = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

      // Ensure term is provided (required)
      if (term && typeof term === 'string') {
        // Ensure parameters are provided (required)
        if (parameters && _typeof(parameters) === 'object' && !Array.isArray(parameters)) {
          var url = "".concat(this.options.serviceUrl, "/autocomplete/").concat(helpers.ourEncodeURIComponent(term), "/search?");
          var queryParams = {};
          var original_query = parameters.original_query,
              group_id = parameters.group_id,
              display_name = parameters.display_name;

          if (original_query) {
            queryParams.original_query = original_query;
          }

          if (group_id) {
            queryParams.group = {
              group_id: group_id,
              display_name: display_name
            };
          }

          this.requests.queue("".concat(url).concat(applyParamsAsString(queryParams, this.options)), undefined, undefined, networkParameters);
          this.requests.send();
          return true;
        }

        this.requests.send();
        return new Error('parameters are required of type object');
      }

      this.requests.send();
      return new Error('term is a required parameter of type string');
    }
    /**
     * Send search results loaded event to API
     *
     * @function trackSearchResultsLoaded
     * @param {string} term - Search results query term
     * @param {object} parameters - Additional parameters to be sent with request
     * @param {number} parameters.num_results - Number of search results in total
     * @param {array} [parameters.item_ids] - List of product item unique identifiers in search results listing
     * @param {object} [networkParameters] - Parameters relevant to the network request
     * @param {number} [networkParameters.timeout] - Request timeout (in milliseconds)
     * @returns {(true|Error)}
     * @description User viewed a search product listing page
     * @example
     * constructorio.tracker.trackSearchResultsLoaded('T-Shirt', {
     *   num_results: 167,
     *   item_ids: ['KMH876', 'KMH140', 'KMH437'],
     * });
     */

  }, {
    key: "trackSearchResultsLoaded",
    value: function trackSearchResultsLoaded(term, parameters) {
      var networkParameters = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

      // Ensure term is provided (required)
      if (term && typeof term === 'string') {
        // Ensure parameters are provided (required)
        if (parameters && _typeof(parameters) === 'object' && !Array.isArray(parameters)) {
          var url = "".concat(this.options.serviceUrl, "/behavior?");
          var queryParams = {
            action: 'search-results',
            term: term
          };
          var num_results = parameters.num_results,
              customer_ids = parameters.customer_ids,
              item_ids = parameters.item_ids;

          if (!helpers.isNil(num_results)) {
            queryParams.num_results = num_results;
          } // Ensure support for both item_ids and customer_ids as parameters


          if (item_ids && Array.isArray(item_ids)) {
            queryParams.customer_ids = item_ids.join(',');
          } else if (customer_ids && Array.isArray(customer_ids)) {
            queryParams.customer_ids = customer_ids.join(',');
          }

          this.requests.queue("".concat(url).concat(applyParamsAsString(queryParams, this.options)), undefined, undefined, networkParameters);
          this.requests.send();
          return true;
        }

        this.requests.send();
        return new Error('parameters are required of type object');
      }

      this.requests.send();
      return new Error('term is a required parameter of type string');
    }
    /**
     * Send click through event to API
     *
     * @function trackSearchResultClick
     * @param {string} term - Search results query term
     * @param {object} parameters - Additional parameters to be sent with request
     * @param {string} parameters.item_name - Product item name
     * @param {string} parameters.item_id - Product item unique identifier
     * @param {string} [parameters.variation_id] - Product item variation unique identifier
     * @param {string} [parameters.result_id] - Search result identifier (returned in response from Constructor)
     * @param {object} [networkParameters] - Parameters relevant to the network request
     * @param {number} [networkParameters.timeout] - Request timeout (in milliseconds)
     * @returns {(true|Error)}
     * @description User clicked a result that appeared within a search product listing page
     * @example
     * constructorio.tracker.trackSearchResultClick('T-Shirt', {
     *   item_name: 'Red T-Shirt',
     *   item_id: 'KMH876',
     *   result_id: '019927c2-f955-4020-8b8d-6b21b93cb5a2',
     * });
     */

  }, {
    key: "trackSearchResultClick",
    value: function trackSearchResultClick(term, parameters) {
      var networkParameters = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

      // Ensure term is provided (required)
      if (term && typeof term === 'string') {
        // Ensure parameters are provided (required)
        if (parameters && _typeof(parameters) === 'object' && !Array.isArray(parameters)) {
          var url = "".concat(this.options.serviceUrl, "/autocomplete/").concat(helpers.ourEncodeURIComponent(term), "/click_through?");
          var queryParams = {};
          var item_name = parameters.item_name,
              name = parameters.name,
              item_id = parameters.item_id,
              customer_id = parameters.customer_id,
              variation_id = parameters.variation_id,
              result_id = parameters.result_id; // Ensure support for both item_name and name as parameters

          if (item_name) {
            queryParams.name = item_name;
          } else if (name) {
            queryParams.name = name;
          } // Ensure support for both item_id and customer_id as parameters


          if (item_id) {
            queryParams.customer_id = item_id;
          } else if (customer_id) {
            queryParams.customer_id = customer_id;
          }

          if (variation_id) {
            queryParams.variation_id = variation_id;
          }

          if (result_id) {
            queryParams.result_id = result_id;
          }

          this.requests.queue("".concat(url).concat(applyParamsAsString(queryParams, this.options)), undefined, undefined, networkParameters);
          this.requests.send();
          return true;
        }

        this.requests.send();
        return new Error('parameters are required of type object');
      }

      this.requests.send();
      return new Error('term is a required parameter of type string');
    }
    /**
     * Send conversion event to API
     *
     * @function trackConversion
     * @param {string} term - Search results query term
     * @param {object} parameters - Additional parameters to be sent with request
     * @param {string} parameters.item_id - Product item unique identifier
     * @param {string} parameters.revenue - Revenue (price) of product item
     * @param {string} [parameters.item_name] - Product item name
     * @param {string} [parameters.variation_id] - Product item variation unique identifier
     * @param {string} [parameters.type='add_to_cart'] - Conversion type
     * @param {boolean} [parameters.is_custom_type] - Specify if type is custom conversion type
     * @param {string} [parameters.display_name] - Display name for the custom conversion type
     * @param {string} [parameters.result_id] - Result identifier (returned in response from Constructor)
     * @param {string} [parameters.section] - Index section
     * @param {object} [networkParameters] - Parameters relevant to the network request
     * @param {number} [networkParameters.timeout] - Request timeout (in milliseconds)
     * @returns {(true|Error)}
     * @description User performed an action indicating interest in an item (add to cart, add to wishlist, etc.)
     * @example
     * constructorio.tracker.trackConversion('T-Shirt', {
     *   item_id: 'KMH876',
     *   revenue: 12.00,
     *   item_name: 'Red T-Shirt',
     *   variation_id: 'KMH879-7632',
     *   type: 'like',
     *   result_id: '019927c2-f955-4020-8b8d-6b21b93cb5a2',
     *   section: 'Products',
     * });
     */

  }, {
    key: "trackConversion",
    value: function trackConversion(term, parameters) {
      var networkParameters = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

      // Ensure parameters are provided (required)
      if (parameters && _typeof(parameters) === 'object' && !Array.isArray(parameters)) {
        var searchTerm = helpers.ourEncodeURIComponent(term) || 'TERM_UNKNOWN';
        var requestPath = "".concat(this.options.serviceUrl, "/v2/behavioral_action/conversion?");
        var queryParams = {};
        var bodyParams = {};
        var name = parameters.name,
            item_name = parameters.item_name,
            item_id = parameters.item_id,
            customer_id = parameters.customer_id,
            variation_id = parameters.variation_id,
            revenue = parameters.revenue,
            _parameters$section = parameters.section,
            section = _parameters$section === void 0 ? 'Products' : _parameters$section,
            display_name = parameters.display_name,
            type = parameters.type,
            is_custom_type = parameters.is_custom_type; // Ensure support for both item_id and customer_id as parameters

        if (item_id) {
          bodyParams.item_id = item_id;
        } else if (customer_id) {
          bodyParams.item_id = customer_id;
        } // Ensure support for both item_name and name as parameters


        if (item_name) {
          bodyParams.item_name = item_name;
        } else if (name) {
          bodyParams.item_name = name;
        }

        if (variation_id) {
          bodyParams.variation_id = variation_id;
        }

        if (revenue) {
          bodyParams.revenue = revenue.toString();
        }

        if (section) {
          queryParams.section = section;
          bodyParams.section = section;
        }

        if (searchTerm) {
          bodyParams.search_term = searchTerm;
        }

        if (type) {
          bodyParams.type = type;
        }

        if (is_custom_type) {
          bodyParams.is_custom_type = is_custom_type;
        }

        if (display_name) {
          bodyParams.display_name = display_name;
        }

        var requestURL = "".concat(requestPath).concat(applyParamsAsString(queryParams, this.options));
        var requestMethod = 'POST';
        var requestBody = applyParams(bodyParams, _objectSpread(_objectSpread({}, this.options), {}, {
          requestMethod: requestMethod
        }));
        this.requests.queue(requestURL, requestMethod, requestBody, networkParameters);
        this.requests.send();
        return true;
      }

      this.requests.send();
      return new Error('parameters are required of type object');
    }
    /**
     * Send purchase event to API
     *
     * @function trackPurchase
     * @param {object} parameters - Additional parameters to be sent with request
     * @param {array} parameters.items - List of product item objects
     * @param {number} parameters.revenue - Revenue
     * @param {string} [parameters.order_id] - Unique order identifier
     * @param {string} [parameters.section] - Index section
     * @param {object} [networkParameters] - Parameters relevant to the network request
     * @param {number} [networkParameters.timeout] - Request timeout (in milliseconds)
     * @returns {(true|Error)}
     * @description User completed an order (usually fired on order confirmation page)
     * @example
     * constructorio.tracker.trackPurchase({
     *   items: [{ item_id: 'KMH876' }, { item_id: 'KMH140' }],
     *   revenue: 12.00,
     *   order_id: 'OUNXBG2HMA',
     *   section: 'Products',
     * });
     */

  }, {
    key: "trackPurchase",
    value: function trackPurchase(parameters) {
      var networkParameters = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      // Ensure parameters are provided (required)
      if (parameters && _typeof(parameters) === 'object' && !Array.isArray(parameters)) {
        var requestPath = "".concat(this.options.serviceUrl, "/v2/behavioral_action/purchase?");
        var queryParams = {};
        var bodyParams = {};
        var items = parameters.items,
            revenue = parameters.revenue,
            order_id = parameters.order_id,
            section = parameters.section;

        if (order_id) {
          // Don't send another purchase event if we have already tracked the order
          if (helpers.hasOrderIdRecord(order_id)) {
            return false;
          }

          helpers.addOrderIdRecord(order_id); // Add order_id to the tracking params

          bodyParams.order_id = order_id;
        }

        if (items && Array.isArray(items)) {
          bodyParams.items = items;
        }

        if (revenue) {
          bodyParams.revenue = revenue;
        }

        if (section) {
          queryParams.section = section;
        } else {
          queryParams.section = 'Products';
        }

        var requestURL = "".concat(requestPath).concat(applyParamsAsString(queryParams, this.options));
        var requestMethod = 'POST';
        var requestBody = applyParams(bodyParams, _objectSpread(_objectSpread({}, this.options), {}, {
          requestMethod: requestMethod
        }));
        this.requests.queue(requestURL, requestMethod, requestBody, networkParameters);
        this.requests.send();
        return true;
      }

      this.requests.send();
      return new Error('parameters are required of type object');
    }
    /**
     * Send recommendation view event to API
     *
     * @function trackRecommendationView
     * @param {object} parameters - Additional parameters to be sent with request
     * @param {string} parameters.url - Current page URL
     * @param {string} parameters.pod_id - Pod identifier
     * @param {number} parameters.num_results_viewed - Number of results viewed
     * @param {number} [parameters.result_count] - Number of results displayed
     * @param {number} [parameters.result_page] - Page number of results
     * @param {string} [parameters.result_id] - Recommendation result identifier (returned in response from Constructor)
     * @param {string} [parameters.section="Products"] - Results section
     * @param {object} [networkParameters] - Parameters relevant to the network request
     * @param {number} [networkParameters.timeout] - Request timeout (in milliseconds)
     * @returns {(true|Error)}
     * @description User viewed a set of recommendations
     * @example
     * constructorio.tracker.trackRecommendationView({
     *   result_count: 22,
     *   result_page: 2,
     *   result_id: '019927c2-f955-4020-8b8d-6b21b93cb5a2',
     *   url: 'https://demo.constructor.io/sandbox/farmstand',
     *   pod_id: '019927c2-f955-4020',
     *   num_results_viewed: 3,
     * });
     */

  }, {
    key: "trackRecommendationView",
    value: function trackRecommendationView(parameters) {
      var networkParameters = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      // Ensure parameters are provided (required)
      if (parameters && _typeof(parameters) === 'object' && !Array.isArray(parameters)) {
        var requestPath = "".concat(this.options.serviceUrl, "/v2/behavioral_action/recommendation_result_view?");
        var bodyParams = {};
        var result_count = parameters.result_count,
            result_page = parameters.result_page,
            result_id = parameters.result_id,
            section = parameters.section,
            url = parameters.url,
            pod_id = parameters.pod_id,
            num_results_viewed = parameters.num_results_viewed;

        if (!helpers.isNil(result_count)) {
          bodyParams.result_count = result_count;
        }

        if (!helpers.isNil(result_page)) {
          bodyParams.result_page = result_page;
        }

        if (result_id) {
          bodyParams.result_id = result_id;
        }

        if (section) {
          bodyParams.section = section;
        } else {
          bodyParams.section = 'Products';
        }

        if (url) {
          bodyParams.url = url;
        }

        if (pod_id) {
          bodyParams.pod_id = pod_id;
        }

        if (!helpers.isNil(num_results_viewed)) {
          bodyParams.num_results_viewed = num_results_viewed;
        }

        var requestURL = "".concat(requestPath).concat(applyParamsAsString({}, this.options));
        var requestMethod = 'POST';
        var requestBody = applyParams(bodyParams, _objectSpread(_objectSpread({}, this.options), {}, {
          requestMethod: requestMethod
        }));
        this.requests.queue(requestURL, requestMethod, requestBody, networkParameters);
        this.requests.send();
        return true;
      }

      this.requests.send();
      return new Error('parameters are required of type object');
    }
    /**
     * Send recommendation click event to API
     *
     * @function trackRecommendationClick
     * @param {object} parameters - Additional parameters to be sent with request
     * @param {string} parameters.pod_id - Pod identifier
     * @param {string} parameters.strategy_id - Strategy identifier
     * @param {string} parameters.item_id - Product item unique identifier
     * @param {string} [parameters.variation_id] - Product item variation unique identifier
     * @param {string} [parameters.section="Products"] - Index section
     * @param {string} [parameters.result_id] - Recommendation result identifier (returned in response from Constructor)
     * @param {number} [parameters.result_count] - Number of results displayed
     * @param {number} [parameters.result_page] - Page number of results
     * @param {number} [parameters.result_position_on_page] - Position of result on page
     * @param {number} [parameters.num_results_per_page] - Number of results on page
     * @param {object} [networkParameters] - Parameters relevant to the network request
     * @param {number} [networkParameters.timeout] - Request timeout (in milliseconds)
     * @returns {(true|Error)}
     * @description User clicked an item that appeared within a list of recommended results
     * @example
     * constructorio.tracker.trackRecommendationClick({
     *   variation_id: 'KMH879-7632',
     *   result_id: '019927c2-f955-4020-8b8d-6b21b93cb5a2',
     *   result_count: 22,
     *   result_page: 2,
     *   result_position_on_page: 2,
     *   num_results_per_page: 12,
     *   pod_id: '019927c2-f955-4020',
     *   strategy_id: 'complimentary',
     *   item_id: 'KMH876',
     * });
     */

  }, {
    key: "trackRecommendationClick",
    value: function trackRecommendationClick(parameters) {
      var networkParameters = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      // Ensure parameters are provided (required)
      if (parameters && _typeof(parameters) === 'object' && !Array.isArray(parameters)) {
        var requestPath = "".concat(this.options.serviceUrl, "/v2/behavioral_action/recommendation_result_click?");
        var bodyParams = {};
        var variation_id = parameters.variation_id,
            section = parameters.section,
            result_id = parameters.result_id,
            result_count = parameters.result_count,
            result_page = parameters.result_page,
            result_position_on_page = parameters.result_position_on_page,
            num_results_per_page = parameters.num_results_per_page,
            pod_id = parameters.pod_id,
            strategy_id = parameters.strategy_id,
            item_id = parameters.item_id;

        if (variation_id) {
          bodyParams.variation_id = variation_id;
        }

        if (section) {
          bodyParams.section = section;
        } else {
          bodyParams.section = 'Products';
        }

        if (result_id) {
          bodyParams.result_id = result_id;
        }

        if (!helpers.isNil(result_count)) {
          bodyParams.result_count = result_count;
        }

        if (!helpers.isNil(result_page)) {
          bodyParams.result_page = result_page;
        }

        if (!helpers.isNil(result_position_on_page)) {
          bodyParams.result_position_on_page = result_position_on_page;
        }

        if (!helpers.isNil(num_results_per_page)) {
          bodyParams.num_results_per_page = num_results_per_page;
        }

        if (pod_id) {
          bodyParams.pod_id = pod_id;
        }

        if (strategy_id) {
          bodyParams.strategy_id = strategy_id;
        }

        if (item_id) {
          bodyParams.item_id = item_id;
        }

        var requestURL = "".concat(requestPath).concat(applyParamsAsString({}, this.options));
        var requestMethod = 'POST';
        var requestBody = applyParams(bodyParams, _objectSpread(_objectSpread({}, this.options), {}, {
          requestMethod: requestMethod
        }));
        this.requests.queue(requestURL, requestMethod, requestBody, networkParameters);
        this.requests.send();
        return true;
      }

      this.requests.send();
      return new Error('parameters are required of type object');
    }
    /**
     * Send browse results loaded event to API
     *
     * @function trackBrowseResultsLoaded
     * @param {object} parameters - Additional parameters to be sent with request
     * @param {string} parameters.url - Current page URL
     * @param {string} parameters.filter_name - Filter name
     * @param {string} parameters.filter_value - Filter value
     * @param {string} [parameters.section="Products"] - Index section
     * @param {number} [parameters.result_count] - Number of results displayed
     * @param {number} [parameters.result_page] - Page number of results
     * @param {string} [parameters.result_id] - Browse result identifier (returned in response from Constructor)
     * @param {string} [parameters.selected_filters] -  Selected filters
     * @param {string} [parameters.sort_order] - Sort order ('ascending' or 'descending')
     * @param {string} [parameters.sort_by] - Sorting method
     * @param {array} [parameters.items] - List of product item objects
     * @param {object} [networkParameters] - Parameters relevant to the network request
     * @param {number} [networkParameters.timeout] - Request timeout (in milliseconds)
     * @returns {(true|Error)}
     * @description User viewed a browse product listing page
     * @example
     * constructorio.tracker.trackBrowseResultsLoaded({
     *   result_count: 22,
     *   result_page: 2,
     *   result_id: '019927c2-f955-4020-8b8d-6b21b93cb5a2',
     *   selected_filters: [ 'brand', 'color' ],
     *   sort_order: 'ascending',
     *   sort_by: 'price',
     *   items: [{ item_id: 'KMH876' }, { item_id: 'KMH140' }],
     *   url: 'https://demo.constructor.io/sandbox/farmstand',
     *   filter_name: 'brand',
     *   filter_value: 'XYZ',
     * });
     */

  }, {
    key: "trackBrowseResultsLoaded",
    value: function trackBrowseResultsLoaded(parameters) {
      var networkParameters = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      // Ensure parameters are provided (required)
      if (parameters && _typeof(parameters) === 'object' && !Array.isArray(parameters)) {
        var requestPath = "".concat(this.options.serviceUrl, "/v2/behavioral_action/browse_result_load?");
        var bodyParams = {};
        var section = parameters.section,
            result_count = parameters.result_count,
            result_page = parameters.result_page,
            result_id = parameters.result_id,
            selected_filters = parameters.selected_filters,
            url = parameters.url,
            sort_order = parameters.sort_order,
            sort_by = parameters.sort_by,
            filter_name = parameters.filter_name,
            filter_value = parameters.filter_value,
            items = parameters.items;

        if (section) {
          bodyParams.section = section;
        } else {
          bodyParams.section = 'Products';
        }

        if (!helpers.isNil(result_count)) {
          bodyParams.result_count = result_count;
        }

        if (!helpers.isNil(result_page)) {
          bodyParams.result_page = result_page;
        }

        if (result_id) {
          bodyParams.result_id = result_id;
        }

        if (selected_filters) {
          bodyParams.selected_filters = selected_filters;
        }

        if (url) {
          bodyParams.url = url;
        }

        if (sort_order) {
          bodyParams.sort_order = sort_order;
        }

        if (sort_by) {
          bodyParams.sort_by = sort_by;
        }

        if (filter_name) {
          bodyParams.filter_name = filter_name;
        }

        if (filter_value) {
          bodyParams.filter_value = filter_value;
        }

        if (items && Array.isArray(items)) {
          bodyParams.items = items;
        }

        var requestURL = "".concat(requestPath).concat(applyParamsAsString({}, this.options));
        var requestMethod = 'POST';
        var requestBody = applyParams(bodyParams, _objectSpread(_objectSpread({}, this.options), {}, {
          requestMethod: requestMethod
        }));
        this.requests.queue(requestURL, requestMethod, requestBody, networkParameters);
        this.requests.send();
        return true;
      }

      this.requests.send();
      return new Error('parameters are required of type object');
    }
    /**
     * Send browse result click event to API
     *
     * @function trackBrowseResultClick
     * @param {object} parameters - Additional parameters to be sent with request
     * @param {string} parameters.filter_name - Filter name
     * @param {string} parameters.filter_value - Filter value
     * @param {string} parameters.item_id - Product item unique identifier
     * @param {string} [parameters.section="Products"] - Index section
     * @param {string} [parameters.variation_id] - Product item variation unique identifier
     * @param {string} [parameters.result_id] - Browse result identifier (returned in response from Constructor)
     * @param {number} [parameters.result_count] - Number of results displayed
     * @param {number} [parameters.result_page] - Page number of results
     * @param {number} [parameters.result_position_on_page] - Position of clicked item
     * @param {number} [parameters.num_results_per_page] - Number of results shown
     * @param {string} [parameters.selected_filters] -  Selected filters
     * @param {object} [networkParameters] - Parameters relevant to the network request
     * @param {number} [networkParameters.timeout] - Request timeout (in milliseconds)
     * @returns {(true|Error)}
     * @description User clicked a result that appeared within a browse product listing page
     * @example
     * constructorio.tracker.trackBrowseResultClick({
     *   variation_id: 'KMH879-7632',
     *   result_id: '019927c2-f955-4020-8b8d-6b21b93cb5a2',
     *   result_count: 22,
     *   result_page: 2,
     *   result_position_on_page: 2,
     *   num_results_per_page: 12,
     *   selected_filters: [ 'brand', 'color' ],
     *   filter_name: 'brand',
     *   filter_value: 'XYZ',
     *   item_id: 'KMH876',
     * });
     */

  }, {
    key: "trackBrowseResultClick",
    value: function trackBrowseResultClick(parameters) {
      var networkParameters = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      // Ensure parameters are provided (required)
      if (parameters && _typeof(parameters) === 'object' && !Array.isArray(parameters)) {
        var requestPath = "".concat(this.options.serviceUrl, "/v2/behavioral_action/browse_result_click?");
        var bodyParams = {};
        var section = parameters.section,
            variation_id = parameters.variation_id,
            result_id = parameters.result_id,
            result_count = parameters.result_count,
            result_page = parameters.result_page,
            result_position_on_page = parameters.result_position_on_page,
            num_results_per_page = parameters.num_results_per_page,
            selected_filters = parameters.selected_filters,
            filter_name = parameters.filter_name,
            filter_value = parameters.filter_value,
            item_id = parameters.item_id;

        if (section) {
          bodyParams.section = section;
        } else {
          bodyParams.section = 'Products';
        }

        if (variation_id) {
          bodyParams.variation_id = variation_id;
        }

        if (result_id) {
          bodyParams.result_id = result_id;
        }

        if (!helpers.isNil(result_count)) {
          bodyParams.result_count = result_count;
        }

        if (!helpers.isNil(result_page)) {
          bodyParams.result_page = result_page;
        }

        if (!helpers.isNil(result_position_on_page)) {
          bodyParams.result_position_on_page = result_position_on_page;
        }

        if (!helpers.isNil(num_results_per_page)) {
          bodyParams.num_results_per_page = num_results_per_page;
        }

        if (selected_filters) {
          bodyParams.selected_filters = selected_filters;
        }

        if (filter_name) {
          bodyParams.filter_name = filter_name;
        }

        if (filter_value) {
          bodyParams.filter_value = filter_value;
        }

        if (item_id) {
          bodyParams.item_id = item_id;
        }

        var requestURL = "".concat(requestPath).concat(applyParamsAsString({}, this.options));
        var requestMethod = 'POST';
        var requestBody = applyParams(bodyParams, _objectSpread(_objectSpread({}, this.options), {}, {
          requestMethod: requestMethod
        }));
        this.requests.queue(requestURL, requestMethod, requestBody, networkParameters);
        this.requests.send();
        return true;
      }

      this.requests.send();
      return new Error('parameters are required of type object');
    }
    /**
     * Send generic result click event to API
     *
     * @function trackGenericResultClick
     * @param {object} parameters - Additional parameters to be sent with request
     * @param {string} parameters.item_id - Product item unique identifier
     * @param {string} [parameters.item_name] - Product item name
     * @param {string} [parameters.variation_id] - Product item variation unique identifier
     * @param {string} [parameters.section="Products"] - Index section
     * @param {object} [networkParameters] - Parameters relevant to the network request
     * @param {number} [networkParameters.timeout] - Request timeout (in milliseconds)
     * @returns {(true|Error)}
     * @description User clicked a result that appeared outside of the scope of search / browse / recommendations
     * @example
     * constructorio.tracker.trackGenericResultClick({
     *   item_id: 'KMH876',
     *   item_name: 'Red T-Shirt',
     *   variation_id: 'KMH879-7632',
     * });
     */

  }, {
    key: "trackGenericResultClick",
    value: function trackGenericResultClick(parameters) {
      var networkParameters = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      // Ensure required parameters are provided
      if (_typeof(parameters) === 'object' && !!parameters.item_id) {
        var requestPath = "".concat(this.options.serviceUrl, "/v2/behavioral_action/result_click?");
        var bodyParams = {};
        var item_id = parameters.item_id,
            item_name = parameters.item_name,
            variation_id = parameters.variation_id,
            section = parameters.section;
        bodyParams.section = section || 'Products';
        bodyParams.item_id = item_id;

        if (item_name) {
          bodyParams.item_name = item_name;
        }

        if (variation_id) {
          bodyParams.variation_id = variation_id;
        }

        var requestURL = "".concat(requestPath).concat(applyParamsAsString({}, this.options));
        var requestMethod = 'POST';
        var requestBody = applyParams(bodyParams, _objectSpread(_objectSpread({}, this.options), {}, {
          requestMethod: requestMethod
        }));
        this.requests.queue(requestURL, requestMethod, requestBody, networkParameters);
        this.requests.send();
        return true;
      }

      this.requests.send();
      return new Error('A parameters object with an "item_id" property is required.');
    }
    /**
     * Subscribe to success or error messages emitted by tracking requests
     *
     * @function on
     * @param {string} messageType - Type of message to listen for ('success' or 'error')
     * @param {function} callback - Callback to be invoked when message received
     * @returns {(true|Error)}
     * @example
     * constructorio.tracker.on('error', (data) => {
     *   // Handle tracking error
     * });
     */

  }, {
    key: "on",
    value: function on(messageType, callback) {
      if (messageType !== 'success' && messageType !== 'error') {
        return new Error('messageType must be a string of value "success" or "error"');
      }

      if (!callback || typeof callback !== 'function') {
        return new Error('callback is required and must be a function');
      }

      this.eventemitter.on(messageType, callback);
      return true;
    }
  }]);

  return Tracker;
}();

module.exports = Tracker;